
var recover = function (element) {
    moveMargin(element, 0);
};

var moveMargin = function (element, movePixels) {
    if (element.moved === movePixels) return;
    element.moved = movePixels;
    element.moving = new Date;
    css(element, {
        marginLeft: movePixels ? movePixels + "px" : "",
        marginRight: movePixels ? -movePixels + "px" : ""
    });
};
var markRowTds = function (tr, deltas, colstart, colend) {
    var inc = 0;
    var collections = [];
    Array.prototype.forEach.call(tr.children, function (td) {
        while (deltas[inc] > 0) {
            deltas[inc++]--;
        }
        var colspan = getColspan(td);
        var rowspan = getRowspan(td);
        rowspan = rowspan > 1 ? rowspan - 1 : 0;
        colspan = colspan > 1 ? colspan - 1 : 0;
        for (var cx = inc, dx = colspan + inc; cx <= dx; cx++) {
            if (!(deltas[cx] >= rowspan)) deltas[cx] = rowspan;
        }
        td.colstart = inc;
        td.colend = dx;
        if (inc >= colstart && dx <= colend) {
            collections.push(td);
        } else if (dx + 1 <= colstart) {
            collections.prev = td;
        } else if (inc - 1 >= colend && !collections.next) {
            collections.next = td;
        }
        inc = dx + 1;
    });
    return collections;
};
var forEachRow = function (tbody, call) {
    for (var tr of tbody.children) {
        if (isTfoot(tr)) continue;
        if (isTableRow(tr)) {
            call(tr);
        }
        else {
            for (var c of tr.children) if (isTableRow(c)) call(c);
        }
    }
}
var getRowsOfTdsByCol = function (table, start, end) {
    var savedCollections = [];
    var thead = getThead(table);
    var tbody = getTbody(table);
    var savedRowDeltas;
    if (thead) savedRowDeltas = [], forEachRow(thead, function (tr) {
        var collections = markRowTds(tr, savedRowDeltas, start, end);
        savedCollections.push(collections);
    });
    if (tbody) savedRowDeltas = [], forEachRow(tbody, function (tr) {
        var collections = markRowTds(tr, savedRowDeltas, start, end);
        savedCollections.push(collections);
    });
    return savedCollections;
}
var getTdsByCol = function (table, start, end) {
    return [].concat.apply([], getRowsOfTdsByCol(table, start, end));
};
var resizeR = function (t) {
    var h = 0;
    var children = [];
    for (var c of t.children) {
        if (getColspan(c) > 1) continue;
        children.push(c);
    }
    for (var c of children) {
        var height = c.offsetHeight;
        if (h < height) h = height;
    }
    for (var c of children) {
        var height = c.offsetHeight;
        if (h > height) {
            css(c, { height: h });
        }
    }
}
var resizeT = function (t, w) {
    if (!w) {
        var w = 0;
        for (var c of t.children) {
            w += c.offsetWidth;
        }
    }
    css(t, { width: w });
}
var getThead = function (table) {
    for (var c of table.children) {
        if (/^thead$/i.test(c.tagName) || c.hasAttribute('thead')) return c;
    }
};
var getTbody = function (table) {
    for (var c of table.children) {
        if (/^tbody$/i.test(c.tagName) || c.hasAttribute("tbody")) return c;
    }
};
var isTfoot = function (c) {
    return /^tfoot$/i.test(c.tagName) || c.hasAttribute("tfoot");
}
var getTExtras = function (table) {
    var tfoot, xb, yb;
    for (var c of table.children) {
        if (isTfoot(c)) {
            tfoot = c;
            continue;
        }
        if (hasClass(c, 'scrollbar')) {
            if (c.hasAttribute('x') || hasClass(c, 'x')) xb = c;
            else yb = c;
        }
    }
    return [tfoot, xb, yb];
};
var isTableRow = function (e) {
    return trElementReg.test(e.tagName);
};
var getRowspan = function (e) {
    return +e.getAttribute('rowspan') || 1;
};
var getColspan = function (e) {
    return +e.getAttribute('colspan') || 1;
}
var resizeColumn = function (target, targetW) {
    var deltaW = targetW - target.offsetWidth;
    forEachRow(this, function (tr) {
        resizeT(tr, tr.offsetWidth + deltaW);
    });
    for (var c of this.children) {
        if (isTfoot(c)) continue;
        if (!isTableRow(c)) {
            var tr = c.querySelector('tr');
            if (!tr) continue;
            c.style.width = tr.style.width;
        }
    }
    var { colstart, colend } = target;
    if (isEmpty(colstart) || isEmpty(colend)) return;
    var ts = getRowsOfTdsByCol(this, colstart, colend);
    for (var cs of ts) {
        var c = cs[cs.length - 1];
        var w = 0;
        for (var c of cs) {
            w += c.offsetWidth;
        }
        if (!cs.length) continue;
        w = targetW - w;
        while (w !== 0) {
            var c = cs.pop();
            var w0 = c.offsetWidth + w;
            if (w0 < 0) {
                w = -w0;
                w0 = w;
            }
            else {
                w = 0;
            }
            if (targetW !== w) css(c, { width: w0 });
        }
    }
};
var resizeTarget = function (event) {
    if (swapping && onclick.preventClick) swapping.end(), swapping = null;
    var { resizing } = this;
    if (!resizing) return;
    event.moveLocked = true;
    var { restX, target } = resizing;
    var targetW = event.clientX - restX;
    if (targetW < 20) targetW = 20;
    resizeColumn.call(this, target, targetW);
    resizing.clientX = event.clientX;
    setFixedColumn.call(this, target);
};
var getFirstSingleColCell = function (table, col) {
    var tds = getTdsByCol(table, col, col);
    while (tds.length) {
        var td = tds.shift();
        var colspan = getColspan(td);
        if (1 === colspan) return td;
    }
}
var adaptTarget = function (event) {
    if (event.which) return;
    var target = event.target;
    while (target && !tdElementReg.test(target.tagName)) {
        target = target.parentNode;
    }
    var result;
    if (target) {
        var position = getScreenPosition(target);
        if (event.clientX - position.left < 7 || position.right - event.clientX < 7) {
            event.preventDefault();
            if (event.clientX - position.left < 7) {
                target = target.previousSibling;
            }
            if (target) target = getFirstSingleColCell(this, target.colend);
            if (target) {
                if (position.right >= getScreenPosition(this).right - 7) {
                    target = this;
                    return;
                }
                css(document.body, { 'cursor': 'col-resize' });
                result = {
                    target,
                    restX: event.clientX - target.offsetWidth
                };
                this.resizing = result;
            }
        }
    }
    if (!result) {
        this.resizing = false;
        css(document.body, { 'cursor': '' });
    }
};
var tdElementReg = /^t[hd]$/i;
var trElementReg = /^tr$/i;
var id = 0;
function enrichField(f) {
    if (!f.id) f.id = ++id;
    if (f.width) {
        f.width_p = fromPixel(f.width);
        return;
    }
    var width;
    if (f.size) {
        width = f.size;
        if (width < 40) width = width * 16;
    }
    else switch (f.type) {
        case "text":
        case "html":
            width = 200;
            break;
        case "input":
            width = 200;
            break;
        case "date":
            width = 180;
        case "datetime":
            width = 200;
            break;
        case "time":
            width = 120;
            break;
        default:
            if (f.options) {
                width = f.options.map(o => o.name instanceof Function ? o.name() : o.name).join(" ").length * 20;
            } else {
                width = String(f.name || f.key).length * 16;
            }
    }
    if (width > 600) width = 600;
    f.width = parseFloat(width) + 60;
    f.width_p = fromPixel(f.width);
    if (!f.key && f.options && isEmpty(f.fixed)) {
        f.fixed = true;
    }
}
var tbodyHeight = function (tbody, hasFoot, width) {
    var rowHeight = calcPixel(36);
    return {
        'max-height': ((innerHeight - (!!hasFoot ? rowHeight : 6) - getScreenPosition(tbody).top - 10) / rowHeight | 0) * rowHeight,
        width
    }
};

var setFixed = function (children, scrolled, left, borderRight) {
    var setBorderRight = function (fixedLeft) {
        var end = fixedLeft[fixedLeft.length - 1];
        if (end && end.style[left]) css(end, {
            [borderRight]: '1px solid #0006'
        });
    };
    var fixedElements = [];
    var offset = 0;
    var fixedWidth = 0;
    for (var c of children) {
        var pc = getScreenPosition(c);
        var isfixed = c.hasAttribute('fixed');
        if (isfixed) {
            if (fixedWidth + scrolled > offset && fixedWidth + pc.width < this.clientWidth / 3) {
                css(c, { [left]: scrolled - offset + fixedWidth, [borderRight]: '' });
                fixedElements.push(c);
                fixedWidth += pc.width;
            }
            else {
                if (c.style[left]) {
                    css(c, { [left]: '', [borderRight]: '' })
                    fixedElements.push(c);
                }
            }
        }
        offset += pc.width;
    }
    setBorderRight(fixedElements);
    for (var f of fixedElements) {
        var cols = getRowsOfTdsByCol(this, f.colstart, f.colend);
        for (var c of cols) css(c[0], {
            [left]: f.style[left],
            [borderRight]: f.style[borderRight]
        });
    }

};
var getTheadChildren = function (table) {
    var thead = getThead(table);
    if (!thead) return;
    if (!isTableRow(thead)) thead = thead.querySelector('tr');
    if (!thead) return;
    return Array.apply(null, thead.children);
}
var fixScroll = function (children) {
    var scrollOnly = !children;
    if (scrollOnly) children = getTheadChildren(this);
    if (!children) return;
    setFixed.call(this, children, this.scrollLeft, 'left', 'borderRight');
    setFixed.call(this, children.reverse(), this.scrollWidth - this.clientWidth - this.scrollLeft, 'right', 'borderLeft');
    var [tfoot, xsb, ysb] = getTExtras(this);
    if (tfoot) {
        css(tfoot, { left: this.scrollLeft });
    }
    if (xsb) {
        if (!scrollOnly) xsb.reshape();
        css(xsb, { left: this.scrollLeft });
    }
    if (ysb) {
        css(ysb, { marginRight: -this.scrollLeft });
    }
};
var setFixedColumn = function (remark) {
    var children = getTheadChildren(this);
    if (!children || children.length < 2) return;
    var cindex = children.length - 1;
    var lastChild = children[cindex];
    if (!lastChild) return;
    if (remark !== false) {
        var scrollWidth = this.clientWidth + this.scrollLeft;
        var restWidth = scrollWidth - lastChild.offsetLeft;
        if (restWidth > 20 || restWidth > lastChild.offsetWidth) {
            var lastStyle = { width: restWidth }
            css(lastChild, lastStyle);
            var thead = getThead(this);
            css(thead, { width: scrollWidth });
            resizeColumn.call(this, lastChild, restWidth);
            var tbody = getTbody(this);
            forEachRow(tbody, function (tr) {
                css(tr.children[cindex], lastStyle);
            });
            css(tbody, { width: thead.offsetWidth });
            remark = true;
        }
    }
    fixScroll.call(this, children);
};
var setClass = function (tds, cls, old) {
    tds.forEach(td => td[cls] = true);
    old.forEach(td => { if (!td[cls]) removeClass(td, cls) });
    tds.forEach(td => { addClass(td, cls); delete td[cls] });
};
var removeYIng = function (activeCols) {
    activeCols.forEach(function (td) {
        removeClass(td, 'y-ing');
    });
};
var removeXIng = function (activeRows) {
    for (var td of activeRows) removeClass(td, 'x-ing');
};

var getTdsOfSameRow = function (td) {
    var tds = [td];
    var tmp = td;
    var rowspan = getRowspan(td);
    var { colstart, colend } = td;
    while (tmp) {
        tmp = tmp.previousElementSibling;
        if (!tmp) break;
        if (colstart - tmp.colend > 1) break;
        if (getRowspan(tmp) > rowspan) break;
        tds.push(tmp);
        colstart = tmp.colstart;
    };
    tmp = td;
    while (tmp) {
        tmp = tmp.nextElementSibling;
        if (!tmp) break;
        if (tmp.colstart - colend > 1) break;
        if (getRowspan(tmp) > rowspan) break;
        tds.push(tmp);
        colend = tmp.colend;
    }
    var tr = td.parentNode;
    while (rowspan > 1) {
        tr = tr.nextElementSibling;
        if (!tr) break;
        for (var c of tr.children) {
            if (c.colstart >= colstart && c.colend <= colend) {
                if (getRowspan(c) <= rowspan) {
                    tds.push(c);
                }
            }
        }
        rowspan--;
    }
    return tds;
};
function setContextMenu(thead) {
    var fields = this.originFields.slice();
    var _do = function () {
        this.checked = !this.checked;
        var f = fields[this.index];
        f.hidden = !this.checked;
        scope.fields = fields.filter(f => !f.hidden);
        var width = thead.scrollWidth / scope.fields.length;
        if (!width || width < 200) width = 200;
        forEachRow(thead, function (tr) {
            for (var td of tr.children) {
                if (td.offsetWidth > width) css(td, { width });
            }
        });
    };
    var menuItems = fields.map((f, i) => ({ name: f.name || "&nbsp", index: i, width: f.width, key: f.key, checked: !f.hidden, do: _do }));
    var scope = this;
    contextmenu(thead, menuItems);
}

var swapping = null;

function table(elem) {
    var tableElement = isElement(elem) ? elem : document.createElement("table");
    var activeCols = [];
    bind('mousemove')(tableElement, adaptTarget);
    var updateSummaryFields = null;
    var thStyles = [];
    var updateStyles = function () {
        thStyles.splice(0, thStyles.length);
        thStyles.push.apply(thStyles, getTheadChildren(table).map(a => {
            return a.getAttribute('style');
        }));
    }
    moveupon(tableElement, {
        start(event) {
            if (this.resizing) return event.preventDefault();
            if (!updateSummaryFields) return;
            var thead = getThead(this);
            var th = getTargetIn(thead, event.target, false);
            if (!th) return;
            if (th.tagName.toLowerCase() === 'tr') th = getTargetIn(th, event.target, false);
            var field = $scoped.get(th).f;
            if (!field || field.fixed || !isHandled(field.key)) return;
            swapping = lazySwap(th, function (value) {
                field.summary = value;
                updateSummaryFields();
            });
            swapping.start();
        },
        move: resizeTarget,
        end() {
            if (swapping) swapping.end(), swapping = null;
            updateStyles();
        }
    });
    var activeRows = [];
    onmousemove(tableElement, function (event) {
        if (table.resizing) return;
        var tbody = getTbody(table);
        a: if (tbody) {
            var tr = getTargetIn(tbody, event.target, false);
            if (!tr) break a;
            var td = getTargetIn(tr, event.target, false);
            if (!td) break a;
            var tds = getTdsOfSameRow(td);
            removeYIng(activeCols);
            setClass(tds, 'x-ing', activeRows);
            activeRows = tds;
            return;
        }
        if (!thead) {
            thead = getThead(table);
        }
        if (!getTargetIn(thead, event.target)) return;
        var tds = cellMatchManager(event.target);

        if (!tds) return;
        setClass(tds, 'y-ing', activeCols);
        removeXIng(activeRows);
        activeCols = tds;
    });
    onmouseleave(tableElement, function () {
        removeYIng(activeCols);
        removeXIng(activeRows);
    });
    var table = tableElement;
    var thead;
    var markedRows = false;
    var markThead = function () {
        var savedRowDeltas = [];
        Array.prototype.forEach.call(thead.children, function (tr) {
            markRowTds(tr, savedRowDeltas);
        });
    };
    var cellMatchManager = function (element) {
        if (!thead) thead = getThead(table);
        var td = getTargetIn(thead, element, false);
        if (!td) return false;
        if (td.tagName.toLowerCase() === 'tr') td = getTargetIn(td, element, false);
        if (!tdElementReg.test(td.tagName)) return false;
        if (!markedRows) {
            markThead();
            markedRows = true;
        }
        var { colstart, colend } = td;
        return getTdsByCol(table, colstart, colend);
    };
    watch(table, {
        showTotal(v) {
            var ts = $scoped.get(this);
            ts.showTotal = v;
            if (v) ts.hasFoot = true;
        }
    })
    table.useIncrease = false;
    var _vbox = function () {
        table.$Left = function (x) {
            if (isFinite(x)) {
                var thd = getTbody(this);
                if (thd.offsetLeft + thd.offsetWidth < x + this.clientWidth) {
                    x = thd.offsetLeft + thd.offsetWidth - this.clientWidth;
                }
                this.scrollLeft = x, fixScroll.call(this);
            }
            else {
                x = this.scrollLeft;
            }
            return x;
        };
        vbox(table, 'x');
    };
    care(table, async function (src) {
        if (src instanceof Table) {
            data = src;
            fields = data.fields;
        }
        else if (src instanceof Array) {
            var [fields, data] = src;
        }
        else throw new Error(i18n`table组件源数据错误！`);
        if (_vbox) _vbox(), _vbox = null;
        watch(table, {
            find(text) {
                if ($scope.data.constructor === Table) {
                    $scope.data.searchText = text;
                    $scope.data.heightLight = marker.wrap;
                    $scope.data.update();
                }
            }
        })
        updateSummaryFields = function () {
            $scope.data.summaryFields = fields.filter(f => f.summary);
            $scope.data.update();
        };
        thead = null;
        fields.forEach(enrichField);
        remove(this.children);
        this.innerHTML = template;
        markedRows = false;
        this.style.display = 'block';
        var $scope = {
            originFields: fields,
            fields: fields.filter(f => !f.hidden),
            isEmpty,
            hasFoot: true,
            thStyles,
            setContextMenu,
            tbody0: null,
            rowClick(d, i, event) {
                active(table, i, d, event.target);
            },
            updateStyles,
            tbody() {
                var e = list.apply(null, arguments);
                css(e, tbodyHeight(e, this.hasFoot));
                css(e, { width: this.adapter.style.width, display: 'block' });
                this.tbody0 = e;
                return e;
            },
            thead(t) {
                var tr = document.createElement('thead');
                tr.$renders = [function () {
                    resizeT(this.firstChild)
                }];
                css(tr, { display: 'block' });
                appendChild(tr, Array.prototype.slice.call(t.children));
                return tr;
            },
            tbodyHeight,
            data,
            adapter: null,
            resizeT,
            resizeR,
            model,
            scrollbar,
            sort(f) {
                this.data.sort(f);
            },
            setWidth(target, f) {
                css(target, { width: f.width });
            },
            a: button,
            setFixedColumn,
            pagination
        };
        render(this, $scope, tableScopes);
        await data;
        if (!data.is_errored) $scope.data = Table.from(fields, data);
        $scope.data.callback = function () {
            render.digest();
        };
    })
    var tableScopes = render.getScopes(table);
    autodragchildren(
        table,
        cellMatchManager,
        function (src, dst, rel, append, parentNode) {
            if (table.src) {
                if (src < 1 || dst < 1) return false;
                var fields = $scoped.get(parentNode).fields;
                var [f] = fields.splice(src - 1, 1);
                fields.splice(dst - 1, 0, f);
            }
            markedRows = false;
            var children = parentNode.children;
            var srcElement = children[src];
            var dstElement = children[rel];
            var dstTds = getRowsOfTdsByCol(table, dstElement.colstart, dstElement.colend);
            var srcTds = getRowsOfTdsByCol(table, srcElement.colstart, srcElement.colend);
            var clearCss = { left: '', right: '', borderLeft: '', borderRight: '' };
            if (append === appendChild.before) {
                srcTds.map(function (src, cx) {
                    var dst = dstTds[cx];
                    var d = dst[0] || dst.next;
                    if (d) src.map(function (s) {
                        css(s, clearCss);
                        append(d, s);
                    });
                    else if (d = dst.prev) src.map(function (s) {
                        css(s, clearCss);
                        appendChild.after(d, s);
                        d = s;
                    });
                })
            } else {
                srcTds.map(function (src, cx) {
                    var dst = dstTds[cx];
                    var d = dst[dst.length - 1] || dst.prev;
                    if (d) src.map(function (s) {
                        css(s, clearCss);
                        append(d, s);
                        d = s;
                    });
                    else if (d = dst.next) src.map(function (s) {
                        css(s, clearCss);
                        appendChild.before(d, s);
                    });
                });

            }
            markThead();
            markedRows = true;
        }
    );
    resizingList.set(table, setFixedColumn);
    return table;
}
